@@ -468,7 +468,7 @@ setClass("Partitioning",
     contains=c("GroupingRanges", "ManyToOneGrouping"),
     representation(
         "VIRTUAL",
-        NAMES="characterORNULL"  # R doesn't like @names !!
+        NAMES="character_OR_NULL"  # R doesn't like @names !!
     ),
     prototype(
         NAMES=NULL
@@ -682,7 +682,7 @@ PartitioningByEnd <- function(x=integer(0), NG=NULL, names=NULL)
 setAs("Ranges", "PartitioningByEnd",
     function(from)
     {
-        ans <- PartitioningByEnd(end(from), names(from))
+        ans <- PartitioningByEnd(end(from), names=names(from))
         if (!identical(start(ans), start(from)))
             stop(wmsg("the Ranges object to coerce does not represent ",
                       "a partitioning"))
