@@ -100,7 +100,7 @@ setAs("NCList", "IRanges", function(from) ranges(from, use.mcols=TRUE))
     unlisted_groups <- unlist(x_groups, use.names=FALSE)
     circle_len <- rep.int(NA_integer_, length(x))
     circle_len[unlisted_groups + 1L] <-
-        rep.int(circle.length, elementLengths(x_groups))
+        rep.int(circle.length, elementNROWS(x_groups))
     .shift_ranges_to_first_circle(x, circle_len)
 }
 
@@ -122,7 +122,7 @@ setAs("NCList", "IRanges", function(from) ranges(from, use.mcols=TRUE))
 .nclist <- function(x_start, x_end, x_subset=NULL)
 {
     nclist_xp <- .NCList_xp(x_start, x_end, x_subset)
-    .Call2("new_NCListSXP_from_NCList", nclist_xp, PACKAGE="IRanges")
+    .Call2("new_NCListAsINTSXP_from_NCList", nclist_xp, PACKAGE="IRanges")
 }
 
 NCList <- function(x, circle.length=NA_integer_)
@@ -149,8 +149,8 @@ print_NCList <- function(x)
 {
     if (!is(x, "NCList"))
         stop("'x' must be an NCList object")
-    .Call2("NCListSXP_print", x@nclist, start(x@ranges), end(x@ranges),
-                              PACKAGE="IRanges")
+    .Call2("NCListAsINTSXP_print", x@nclist, start(x@ranges), end(x@ranges),
+                                   PACKAGE="IRanges")
     invisible(NULL)
 }
 
@@ -161,17 +161,24 @@ print_NCList <- function(x)
 
 ### NOT exported.
 findOverlaps_NCList <- function(query, subject,
-             min.score=1L,
+             maxgap=0L, minoverlap=1L,
              type=c("any", "start", "end", "within", "extend", "equal"),
              select=c("all", "first", "last", "arbitrary", "count"),
              circle.length=NA_integer_)
 {
     if (!(is(query, "Ranges") && is(subject, "Ranges")))
         stop("'query' and 'subject' must be Ranges objects")
-    if (!isSingleNumber(min.score))
-        stop("'min.score' must be a single integer")
-    if (!is.integer(min.score))
-        min.score <- as.integer(min.score)
+
+    if (!isSingleNumber(maxgap))
+        stop("'maxgap' must be a single integer")
+    if (!is.integer(maxgap))
+        maxgap <- as.integer(maxgap)
+
+    if (!isSingleNumber(minoverlap))
+        stop("'minoverlap' must be a single integer")
+    if (!is.integer(minoverlap))
+        minoverlap <- as.integer(minoverlap)
+
     type <- match.arg(type)
     select <- match.arg(select)
     circle.length <- .normarg_circle.length1(circle.length)
@@ -195,48 +202,10 @@ findOverlaps_NCList <- function(query, subject,
            start(query), end(query),
            start(subject), end(subject),
            nclist, nclist_is_q,
-           min.score, type, select, circle.length,
+           maxgap, minoverlap, type, select, circle.length,
            PACKAGE="IRanges")
 }
 
-### NOT exported.
-min_overlap_score <- function(maxgap=0L, minoverlap=1L, type="any")
-{
-    ## Check and normalize 'maxgap'.
-    if (!isSingleNumber(maxgap))
-        stop("'maxgap' must be a single integer")
-    if (!is.integer(maxgap))
-        maxgap <- as.integer(maxgap)
-
-    ## Check and normalize 'minoverlap'.
-    if (!isSingleNumber(minoverlap))
-        stop("'minoverlap' must be a single integer")
-    if (!is.integer(minoverlap))
-        minoverlap <- as.integer(minoverlap)
-
-    if (maxgap < 0L)
-        stop("'maxgap' cannot be negative")
-    if (minoverlap < 0L)
-        stop("'minoverlap' cannot be negative")
-    if (maxgap != 0L) {
-        if (minoverlap > 1L)
-            stop("'minoverlap' must be <= 1 when 'maxgap' is not 0")
-        if (type %in% c("within", "equal")) {
-            warning(wmsg(
-                "With the new findOverlaps/countOverlaps implementation based ",
-                "on Nested Containment Lists, 'maxgap' has no special meaning ",
-                "when 'type' is set to \"within\" or \"equal\". Note that ",
-                "this is a change with respect to the old implementation ",
-                "based on Interval Trees. See ?NCList for more information ",
-                "about this change and other differences between the new and ",
-                "old implementations. Use the 'algorithm' argument to switch ",
-                "between the 2 implementations."
-            ))
-        }
-    }
-    minoverlap - maxgap
-}
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Representation of a list of NCList objects
@@ -278,7 +247,7 @@ setMethod("start", "NCLists", function(x, ...) start(ranges(x)))
 setMethod("end", "NCLists", function(x, ...) end(ranges(x)))
 setMethod("width", "NCLists", function(x) width(ranges(x)))
 
-setMethod("elementLengths", "NCLists", function(x) elementLengths(ranges(x)))
+setMethod("elementNROWS", "NCLists", function(x) elementNROWS(ranges(x)))
 setMethod("getListElement", "NCLists",
     function (x, i, exact=TRUE)
     {
@@ -344,10 +313,13 @@ setAs("RangesList", "NCLists", function(from) NCLists(from))
 ### NOT exported. Workhorse behind findOverlaps_NCLists() below and behind
 ### GenomicRanges:::findOverlaps_GNCList().
 NCList_find_overlaps_in_groups <- function(
-                        q, q_space, q_groups,
-                        s, s_space, s_groups,
-                        nclists, nclist_is_q,
-                        min.score, type, select, circle.length)
+             q, q_space, q_groups,
+             s, s_space, s_groups,
+             nclists, nclist_is_q,
+             maxgap=0L, minoverlap=1L,
+             type=c("any", "start", "end", "within", "extend", "equal"),
+             select=c("all", "first", "last", "arbitrary", "count"),
+             circle.length)
 {
     if (!(is(q, "Ranges") && is(s, "Ranges")))
         stop("'q' and 's' must be Ranges object")
@@ -355,6 +327,20 @@ NCList_find_overlaps_in_groups <- function(
         stop("'q_groups' must be a CompressedIntegerList object")
     if (!is(s_groups, "CompressedIntegerList"))
         stop("'s_groups' must be a CompressedIntegerList object")
+
+    if (!isSingleNumber(maxgap))
+        stop("'maxgap' must be a single integer")
+    if (!is.integer(maxgap))
+        maxgap <- as.integer(maxgap)
+
+    if (!isSingleNumber(minoverlap))
+        stop("'minoverlap' must be a single integer")
+    if (!is.integer(minoverlap))
+        minoverlap <- as.integer(minoverlap)
+
+    type <- match.arg(type)
+    select <- match.arg(select)
+
     q_circle_len <- circle.length
     q_circle_len[which(nclist_is_q)] <- NA_integer_
     q <- .shift_ranges_in_groups_to_first_circle(q, q_groups, q_circle_len)
@@ -365,7 +351,7 @@ NCList_find_overlaps_in_groups <- function(
            start(q), end(q), q_space, q_groups,
            start(s), end(s), s_space, s_groups,
            nclists, nclist_is_q,
-           min.score, type, select, circle.length,
+           maxgap, minoverlap, type, select, circle.length,
            PACKAGE="IRanges")
 }
 
@@ -378,12 +364,12 @@ NCList_find_overlaps_in_groups <- function(
 {
     ## Compute list element lengths and offsets for 'query'.
     query_partitioning <- PartitioningByEnd(query)
-    query_eltlens <- width(query_partitioning)
+    query_eltNROWS <- width(query_partitioning)
     query_offsets <- start(query_partitioning) - 1L
 
     ## Compute list element lengths and offsets for 'subject'.
     subject_partitioning <- PartitioningByEnd(subject)
-    subject_eltlens <- width(subject_partitioning)
+    subject_eltNROWS <- width(subject_partitioning)
     subject_offsets <- start(subject_partitioning) - 1L
 
     if (select != "all") {
@@ -399,32 +385,28 @@ NCList_find_overlaps_in_groups <- function(
     lapply(seq_len(min(length(query), length(subject))),
            function(i) {
                hits <- all_hits[h_skeleton[[i]]]
-               hits@queryHits <- hits@queryHits - query_offsets[[i]]
-               hits@subjectHits <- hits@subjectHits - subject_offsets[[i]]
-               hits@queryLength <- query_eltlens[[i]]
-               hits@subjectLength <- subject_eltlens[[i]]
+               hits@from <- hits@from - query_offsets[[i]]
+               hits@to <- hits@to - subject_offsets[[i]]
+               hits@nLnode <- query_eltNROWS[[i]]
+               hits@nRnode <- subject_eltNROWS[[i]]
                hits
            })
 }
 
 ### NOT exported.
 ### Return an ordinary list of:
-###   (a) Hits objects if 'select' is "all". In that case the list has the
-###       length of the shortest of 'query' or 'subject'.
+###   (a) SortedByQueryHits objects if 'select' is "all". In that case the
+###       list has the length of the shortest of 'query' or 'subject'.
 ###   (b) integer vectors if 'select' is not "all". In that case the list is
 ###       parallel to and has the same shape as 'query'.
 findOverlaps_NCLists <- function(query, subject,
-             min.score=1L,
+             maxgap=0L, minoverlap=1L,
              type=c("any", "start", "end", "within", "extend", "equal"),
              select=c("all", "first", "last", "arbitrary", "count"),
              circle.length=NA_integer_)
 {
     if (!(is(query, "RangesList") && is(subject, "RangesList")))
         stop("'query' and 'subject' must be RangesList objects")
-    if (!isSingleNumber(min.score))
-        stop("'min.score' must be a single integer")
-    if (!is.integer(min.score))
-        min.score <- as.integer(min.score)
     type <- match.arg(type)
     select <- match.arg(select)
     circle.length <- .normarg_circle.length2(circle.length,
@@ -459,7 +441,7 @@ findOverlaps_NCLists <- function(query, subject,
                         q, NULL, q_groups,
                         s, NULL, s_groups,
                         nclists, nclist_is_q,
-                        min.score, type, select, circle.length)
+                        maxgap, minoverlap, type, select, circle.length)
     .split_and_remap_hits(all_hits, query, subject, select)
 }
 
