@@ -18,10 +18,17 @@
 ### NOT exported but used in the GenomicRanges package
 global2local_revmap <- function(unlisted_revmap, y, x)
 {
-    offsets <- rep.int(start(PartitioningByEnd(x)) - 1L, lengths(y))
+    offsets <- rep.int(start(PartitioningByEnd(x)) - 1L, elementNROWS(y))
     unlisted_revmap - offsets
 }
 
+### NOT exported but used in the GenomicFeatures package
+local2global_revmap <- function(unlisted_revmap, y, x)
+{
+    offsets <- rep.int(start(PartitioningByEnd(x)) - 1L, elementNROWS(y))
+    unlisted_revmap + offsets
+}
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### range()
@@ -30,8 +37,10 @@ global2local_revmap <- function(unlisted_revmap, y, x)
 ### Always return an IRanges (or NormalIRanges) *instance* whatever Ranges
 ### derivative the input is, so does NOT act like an endomorphism in general. 
 setMethod("range", "Ranges",
-    function(x, ..., na.rm=FALSE)
+    function(x, ..., with.revmap=FALSE, na.rm=FALSE)
     {
+        if (!isTRUEorFALSE(with.revmap))
+            stop("'with.revmap' must be TRUE or FALSE")
         if (!identical(na.rm, FALSE))
             warning("\"range\" method for Ranges objects ",
                     "ignores the 'na.rm' argument")
@@ -46,21 +55,24 @@ setMethod("range", "Ranges",
         ans <- .Call2("IRanges_range", ir, PACKAGE="IRanges")
         if (is(x, "NormalIRanges"))
             ans <- as(ans, "NormalIRanges")
+        if (with.revmap){
+            mcols(ans) <- DataFrame(revmap=IntegerList(seq_along(ir)))
+        }
         ans
     }
 )
 
 setMethod("range", "RangesList",
-    function(x, ..., na.rm=FALSE)
+    function(x, ..., with.revmap=FALSE, na.rm=FALSE)
     {
         if (length(list(x, ...)) >= 2L)
             x <- merge(x, ...)
-        endoapply(x, range)
+        endoapply(x, range, with.revmap=with.revmap)
     }
 )
 
 ### Equivalent to, but much faster than, 'endoapply(x, range)'.
-.CompressedIRangesList.range <- function(x)
+.range_CompressedIRangesList <- function(x, with.revmap=FALSE)
 {
     ## 'x_start' and 'x_end' are CompressedIntegerList objects with the
     ## same shape as 'x'.
@@ -74,25 +86,29 @@ setMethod("range", "RangesList",
     sv <- Views(x_start@unlistData, x_start@partitioning)
     ev <- Views(x_end@unlistData, x_end@partitioning)
     is_not_empty_view <- width(sv) != 0L  # same as 'width(ev) != 0L'
-    ans_unlistData <- IRanges(viewMins(sv)[is_not_empty_view],
-                              viewMaxs(ev)[is_not_empty_view])
-    ans_partitioning <- new2("PartitioningByEnd",
-                                         end=cumsum(is_not_empty_view),
-                                         check=FALSE)
-    ans <- new2("CompressedIRangesList", unlistData=ans_unlistData,
-                                         partitioning=ans_partitioning,
-                                         check=FALSE)
+    unlisted_ans <- IRanges(viewMins(sv)[is_not_empty_view],
+                            viewMaxs(ev)[is_not_empty_view])
+    ans_partitioning <- PartitioningByEnd(cumsum(is_not_empty_view))
+    if (with.revmap) {
+        x_partitioning <- unname(PartitioningByEnd(x))
+        global_revmap <- relist(seq_along(unlist(x, use.names=FALSE)),
+                                x_partitioning[width(x_partitioning) != 0L])
+        local_revmap <- global2local_revmap(global_revmap, ans_partitioning, x)
+        mcols(unlisted_ans)$revmap <- local_revmap
+    }
+    ans <- relist(unlisted_ans, ans_partitioning)
     names(ans) <- names(x)
-    mcols(ans) <- mcols(x)
     ans
 }
 
 setMethod("range", "CompressedIRangesList",
-    function(x, ..., na.rm=FALSE)
+    function(x, ..., with.revmap=FALSE, na.rm=FALSE)
     {
+        if (!isTRUEorFALSE(with.revmap))
+            stop("'with.revmap' must be TRUE or FALSE")
         if (length(list(x, ...)) >= 2L)
             x <- merge(x, ...)
-        .CompressedIRangesList.range(x)
+        .range_CompressedIRangesList(x, with.revmap=with.revmap)
     }
 )
 
@@ -182,99 +198,38 @@ setMethod("reduce", "RangesList",
 
 ### 'with.inframe.attrib' is ignored for now.
 ### TODO: Support 'with.inframe.attrib=TRUE'.
-setMethod("reduce", "CompressedIRangesList",
-    function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE,
-                with.inframe.attrib=FALSE)
-    {
-        if (!isTRUEorFALSE(drop.empty.ranges))
-            stop("'drop.empty.ranges' must be TRUE or FALSE")
-        if (!isSingleNumber(min.gapwidth))
-            stop("'min.gapwidth' must be a single integer")
-        if (!is.integer(min.gapwidth))
-            min.gapwidth <- as.integer(min.gapwidth)
-        if (min.gapwidth < 0L)
-            stop("'min.gapwidth' must be non-negative")
-        if (!isTRUEorFALSE(with.revmap))
-            stop("'with.revmap' must be TRUE or FALSE")
-        if (!identical(with.inframe.attrib, FALSE))
-            stop("'with.inframe.attrib' argument not yet supported ",
-                 "when reducing a CompressedIRangesList object")
-        C_ans <- .Call2("CompressedIRangesList_reduce",
-                        x, drop.empty.ranges, min.gapwidth,
-                        with.revmap,
-                        PACKAGE="IRanges")
-        ans_unlistData <- new2("IRanges", start=C_ans$start,
-                                          width=C_ans$width,
-                                          check=FALSE)
-        if (with.revmap) {
-            mcols(ans_unlistData) <- DataFrame(revmap=IntegerList(C_ans$revmap))
-        }
-        ans_partitioning <- PartitioningByEnd(C_ans$partitioning_by_end)
-        names(ans_partitioning) <- names(x)
-        relist(ans_unlistData, ans_partitioning)
-    }
-)
+.reduce_CompressedIRangesList <- function(x, drop.empty.ranges=FALSE,
+                                             min.gapwidth=1L,
+                                             with.revmap=FALSE,
+                                             with.inframe.attrib=FALSE)
+{
+    if (!isTRUEorFALSE(drop.empty.ranges))
+        stop("'drop.empty.ranges' must be TRUE or FALSE")
+    if (!isSingleNumber(min.gapwidth))
+        stop("'min.gapwidth' must be a single integer")
+    if (!is.integer(min.gapwidth))
+        min.gapwidth <- as.integer(min.gapwidth)
+    if (min.gapwidth < 0L)
+        stop("'min.gapwidth' must be non-negative")
+    if (!isTRUEorFALSE(with.revmap))
+        stop("'with.revmap' must be TRUE or FALSE")
+    if (!identical(with.inframe.attrib, FALSE))
+        stop("'with.inframe.attrib' argument not yet supported ",
+             "when reducing a CompressedIRangesList object")
+    C_ans <- .Call2("CompressedIRangesList_reduce",
+                    x, drop.empty.ranges, min.gapwidth, with.revmap,
+                    PACKAGE="IRanges")
+    unlisted_ans <- new2("IRanges", start=C_ans$start,
+                                    width=C_ans$width,
+                                    check=FALSE)
+    if (with.revmap)
+        mcols(unlisted_ans) <- DataFrame(revmap=IntegerList(C_ans$revmap))
+    ans_partitioning <- PartitioningByEnd(C_ans$breakpoints)
+    names(ans_partitioning) <- names(x)
+    relist(unlisted_ans, ans_partitioning)
+}
 
-setMethod("reduce", "RangedData",
-          function(x, drop.empty.ranges=FALSE, by=character(),
-                   min.gapwidth=1L, with.inframe.attrib=FALSE)
-          {
-            .Defunct(msg=c("the \"reduce\" method for RangedData object ",
-                           "is defunct"))
-            if (!isTRUEorFALSE(drop.empty.ranges))
-                stop("'drop.empty.ranges' must be TRUE or FALSE")
-            if (!isSingleNumber(min.gapwidth))
-                stop("'min.gapwidth' must be a single integer")
-            if (!is.integer(min.gapwidth))
-                min.gapwidth <- as.integer(min.gapwidth)
-            if (min.gapwidth < 0L)
-                stop("'min.gapwidth' must be non-negative")
-            FUN <- function(y) {
-              name <- names(y)
-              ranges <- ranges(y)[[1L]]
-              values <- values(y)[[1L]]
-              inds <-
-                unname(split(seq_len(nrow(values)), lapply(values, as.vector)))
-              rlist <-
-                lapply(inds, function(i) {
-                         rngs <-
-                           reduce(ranges[i],
-                                  drop.empty.ranges=drop.empty.ranges,
-                                  min.gapwidth=min.gapwidth,
-                                  with.inframe.attrib=with.inframe.attrib)
-                         list(ranges = rngs,
-                              values =
-                              values[rep(i, length.out = length(rngs)), ,
-                                     drop=FALSE])
-                       })
-              ranges <-
-                IRangesList(do.call(c, lapply(rlist, "[[", "ranges")))
-              names(ranges) <- name
-              values <-
-                SplitDataFrameList(do.call(rbind,
-                                           lapply(rlist, "[[", "values")))
-              names(values) <- name
-              new2(class(y), ranges = ranges, values = values, check = FALSE)
-            }
-            if (ncol(x) == 0 || length(by) == 0) {
-              ranges <-
-                reduce(ranges(x),
-                       drop.empty.ranges = drop.empty.ranges,
-                       min.gapwidth = min.gapwidth,
-                       with.inframe.attrib = with.inframe.attrib)
-              listData <-
-                S4Vectors:::make_zero_col_DataFrame(sum(elementNROWS(ranges)))
-              end <- cumsum(elementNROWS(ranges))
-              names(end) <- names(ranges)
-              partitioning <- PartitioningByEnd(end)
-              initialize(x,
-                         ranges = ranges,
-                         values = relist(listData, partitioning))
-            } else {
-              endoapply(x[,by], FUN)
-            }
-          })
+setMethod("reduce", "CompressedIRangesList", .reduce_CompressedIRangesList)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -324,44 +279,77 @@ setMethod("gaps", "Views",
     }
 )
 
-setMethod("gaps", "RangesList",
-    function(x, start = NA, end = NA)
-    {
-        lx <- length(x)
-        if (!S4Vectors:::isNumericOrNAs(start))
-            stop("'start' must be an integer vector or NA")
-        if (!is.integer(start))
-            start <- as.integer(start)
-        if (!S4Vectors:::isNumericOrNAs(end))
-            stop("'end' must be an integer vector or NA")
-        if (!is.integer(end))
-            end <- as.integer(end)
-        start <- IntegerList(as.list(S4Vectors:::recycleVector(start, lx)))
-        end <- IntegerList(as.list(S4Vectors:::recycleVector(end, lx)))
-        mendoapply(gaps, x, start = start, end = end)
-    }
-)
+.gaps_RangesList <- function(x, start=NA, end=NA)
+{
+    x_len <- length(x)
+    if (!S4Vectors:::isNumericOrNAs(start))
+        stop("'start' must be an integer vector or NA")
+    if (!is.integer(start))
+        start <- as.integer(start)
+    if (!S4Vectors:::isNumericOrNAs(end))
+        stop("'end' must be an integer vector or NA")
+    if (!is.integer(end))
+        end <- as.integer(end)
+    start <- IntegerList(as.list(S4Vectors:::recycleVector(start, x_len)))
+    end <- IntegerList(as.list(S4Vectors:::recycleVector(end, x_len)))
+    mendoapply(gaps, x, start = start, end = end)
+}
 
-setMethod("gaps", "CompressedIRangesList",
-    function(x, start = NA, end = NA)
-    {
-        lx <- length(x)
-        if (!S4Vectors:::isNumericOrNAs(start))
-            stop("'start' must be an integer vector or NA")
-        if (!is.integer(start))
-            start <- as.integer(start)
-        if (!S4Vectors:::isNumericOrNAs(end))
-            stop("'end' must be an integer vector or NA")
-        if (!is.integer(end))
-            end <- as.integer(end)
-        if ((length(start) != 1) || (length(end) != 1)) {
-            start <- S4Vectors:::recycleVector(start, lx)
-            end <- S4Vectors:::recycleVector(end, lx)
-        }
-        .Call2("CompressedIRangesList_gaps", x, start, end,
-               PACKAGE="IRanges")
+setMethod("gaps", "RangesList", .gaps_RangesList)
+
+.gaps_CompressedIRangesList <- function(x, start=NA, end=NA)
+{
+    ## Normalize 'start'.
+    if (!S4Vectors:::isNumericOrNAs(start))
+        stop("'start' must be an integer vector or NA")
+    if (!is.integer(start))
+        start <- as.integer(start)
+    if (length(start) != 1L)
+        start <- S4Vectors:::V_recycle(start, x,
+                                       x_what="start", skeleton_what="x")
+    ## Normalize 'end'.
+    if (!S4Vectors:::isNumericOrNAs(end))
+        stop("'end' must be an integer vector or NA")
+    if (!is.integer(end))
+        end <- as.integer(end)
+    if (length(end) != 1L)
+        end <- S4Vectors:::V_recycle(end, x,
+                                     x_what="end", skeleton_what="x")
+
+    chunksize <- 10000000L
+    if (length(x) <= chunksize) {
+        ## Process all at once.
+        ans <- .Call2("CompressedIRangesList_gaps",
+                      x, start, end,
+                      PACKAGE="IRanges")
+        return(ans)
     }
-)
+
+    ## Process by chunk.
+    verbose <- getOption("verbose", default=FALSE)
+    chunks <- as(breakInChunks(length(x), chunksize), "IRanges")
+    ans_chunks <- lapply(seq_along(chunks),
+        function(i) {
+            if (verbose)
+                cat("Processing chunk #", i, "/", length(chunks), " ... ",
+                    sep="")
+            chunk <- chunks[i]
+            x_chunk <- extractROWS(x, chunk)
+            start_chunk <- if (length(start) == 1L) start
+                           else extractROWS(start, chunk)
+            end_chunk <- if (length(end) == 1L) end
+                         else extractROWS(end, chunk)
+            ans_chunk <- .gaps_CompressedIRangesList(x_chunk,
+                                                     start=start_chunk,
+                                                     end=end_chunk)
+            if (verbose)
+                cat("OK\n")
+            ans_chunk
+        })
+    do.call(c, ans_chunks)
+}
+
+setMethod("gaps", "CompressedIRangesList", .gaps_CompressedIRangesList)
 
 ### 'start' and 'end' are ignored.
 setMethod("gaps", "MaskCollection",
@@ -390,6 +378,8 @@ setGeneric("disjoin", function(x, ...) standardGeneric("disjoin"))
 setMethod("disjoin", "Ranges",
     function(x, with.revmap=FALSE)
     {
+        if (!isTRUEorFALSE(with.revmap))
+            stop("'with.revmap' must be TRUE or FALSE")
         ## starts: original starts and end+1 when inside another interval
         ## ends: original ends and start-1 when inside another interval
         starts <- unique(start(x))
@@ -420,6 +410,8 @@ setMethod("disjoin", "RangesList",
 setMethod("disjoin", "CompressedIRangesList",
           function(x, with.revmap=FALSE, ...)
           {
+              if (!isTRUEorFALSE(with.revmap))
+                  stop("'with.revmap' must be TRUE or FALSE")        
               .wunlist <- function(x)
                   ## unlist CompressedIntegerList, with integer(0) as 0
               {
@@ -451,8 +443,9 @@ setMethod("disjoin", "CompressedIRangesList",
               ## localize 'revmap'
               if (with.revmap) {
                   unlisted_ans <- unlist(ans, use.names=FALSE)
-                  mcols(unlisted_ans)$revmap <-
-                      global2local_revmap(mcols(unlisted_ans)$revmap, ans, x)
+                  global_revmap <- mcols(unlisted_ans)$revmap
+                  local_revmap <- global2local_revmap(global_revmap, ans, x)
+                  mcols(unlisted_ans)$revmap <- local_revmap
                   ans <- relist(unlisted_ans, ans)
               }
 
